@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # ===================================================================
 #
 # Copyright (c) 2016, Legrandin <helderijs@gmail.com>
@@ -35,7 +36,6 @@ import binascii
 import struct
 
 from Cryptodome import Random
-from Cryptodome.IO import PKCS8, PEM
 from Cryptodome.Util.py3compat import tobytes, bord, tostr
 from Cryptodome.Util.asn1 import DerSequence
 
@@ -98,6 +98,9 @@ class RsaKey(object):
             raise ValueError("Some RSA components are missing")
         for component, value in kwargs.items():
             setattr(self, "_" + component, value)
+        if input_set == private_set:
+            self._dp = self._d % (self._p - 1)  # = (e⁻¹) mod (p-1)
+            self._dq = self._d % (self._q - 1)  # = (e⁻¹) mod (q-1)
 
     @property
     def n(self):
@@ -140,12 +143,12 @@ class RsaKey(object):
         return (self._n.size_in_bits() - 1) // 8 + 1
 
     def _encrypt(self, plaintext):
-        if not 0 < plaintext < self._n:
+        if not 0 <= plaintext < self._n:
             raise ValueError("Plaintext too large")
         return int(pow(Integer(plaintext), self._e, self._n))
 
     def _decrypt(self, ciphertext):
-        if not 0 < ciphertext < self._n:
+        if not 0 <= ciphertext < self._n:
             raise ValueError("Ciphertext too large")
         if not self.has_private():
             raise TypeError("This is not a private key")
@@ -156,17 +159,14 @@ class RsaKey(object):
         r = Integer.random_range(min_inclusive=1, max_exclusive=self._n)
         # Step 2: Compute c' = c * r**e mod n
         cp = Integer(ciphertext) * pow(r, self._e, self._n) % self._n
-        # Step 3: Compute m' = c'**d mod n       (ordinary RSA decryption)
-        m1 = pow(cp, self._d % (self._p - 1), self._p)
-        m2 = pow(cp, self._d % (self._q - 1), self._q)
-        h = m2 - m1
-        while h < 0:
-            h += self._q
-        h = (h * self._u) % self._q
+        # Step 3: Compute m' = c'**d mod n       (normal RSA decryption)
+        m1 = pow(cp, self._dp, self._p)
+        m2 = pow(cp, self._dq, self._q)
+        h = ((m2 - m1) * self._u) % self._q
         mp = h * self._p + m1
         # Step 4: Compute m = m**(r-1) mod n
         result = (r.inverse(self._n) * mp) % self._n
-        # Verify no faults occured
+        # Verify no faults occurred
         if ciphertext != pow(result, self._e, self._n):
             raise ValueError("Fault detected in RSA decryption")
         return result
@@ -333,6 +333,8 @@ class RsaKey(object):
                 if format == 'DER' and passphrase:
                     raise ValueError("PKCS#1 private key cannot be encrypted")
             else:  # PKCS#8
+                from Cryptodome.IO import PKCS8
+
                 if format == 'PEM' and protection is None:
                     key_type = 'PRIVATE KEY'
                     binary_key = PKCS8.wrap(binary_key, oid, None)
@@ -353,6 +355,8 @@ class RsaKey(object):
         if format == 'DER':
             return binary_key
         if format == 'PEM':
+            from Cryptodome.IO import PEM
+
             pem_str = PEM.encode(binary_key, key_type, passphrase, randfunc)
             return tobytes(pem_str)
 
@@ -652,6 +656,8 @@ def _import_x509_cert(encoded, *kwargs):
 
 
 def _import_pkcs8(encoded, passphrase):
+    from Cryptodome.IO import PKCS8
+
     k = PKCS8.unwrap(encoded, passphrase)
     if k[0] != oid:
         raise ValueError("No PKCS#8 encoded RSA key")
@@ -741,6 +747,8 @@ def import_key(extern_key, passphrase=None):
     .. _`OpenSSH 6.5`: https://flak.tedunangst.com/post/new-openssh-key-format-and-bcrypt-pbkdf
     """
 
+    from Cryptodome.IO import PEM
+
     extern_key = tobytes(extern_key)
     if passphrase is not None:
         passphrase = tobytes(passphrase)
